class BattlesController < ApplicationController
  before_filter :authenticate_user!, :only => [:new, :create, :my_battles, :join, :attack]
  
  def index
    #@battles = Battle.joinable_battles
    @joinable = Battle.joinable.paginate(:page => params[:page])
    @in_progress = Battle.in_progress
    
    json = {:joinable => @joinable, :in_progress => Battle.in_progress, :complete => Battle.complete}
    respond_to do |format|
      format.html
      format.json {render :json => json}
    end
  end
  
  def show
    @battle = Battle.find(params[:id])
    @battle_history = BattleTurn.skill_history(@battle.id)
    
    #(current_user == @battle.opponent) ? role = "opponent" : role = "starter"
    role = ""
    if(current_user == @battle.opponent)
      role = "opponent"
    elsif(current_user == @battle.starter)
      role ="starter"
    else
      role = "guest"
    end
    
    respond_to do |format|
      format.html
      format.json { render :json => {:battle => @battle, :battle_history => @battle_history, :role => role }}
    end 
  end
  
  def new
    @battle = Battle.new()
  end
  
  def create
    @battle = Battle.new(params[:battle])
    @battle.starter = current_user
    @battle.starter_team_member_id = @battle.starter.current_team.first.id
    
    if @battle.save
      redirect_to root_path, :flash => {:success => "Battle created!"}
    else
      render action: "new"
    end
  end
  
=begin  
  def update
    @battle = Battle.find(params[:id])

    if @battle.update_attributes(params[:battle])
      redirect_to @battle, :flash => {:success => "You joined the battle."}
    else
      render action: "join"
    end
  end
=end
  
  def my_battles
    @battles = current_user.battles
    
    respond_to do |format|
      format.html
      format.json {render :json => @battles}
    end
  end
  
  def join
    @battle = Battle.find(params[:id])
    
    if(@battle.starter.id == current_user.id)
      redirect_to root_path, :flash => {:error => "Can't join a battle with yourself!"}
      return
    end
    if(@battle.opponent != nil)
      redirect_to root_path, :flash => {:error => "Battle is full."}
      return
    else
      @battle.opponent = current_user
      @battle.opponent_team_member_id = @battle.opponent.current_team.first.id
      first_turn = BattleTurn.new(:battle_id => @battle.id)
      first_turn.save
    end
    
    if @battle.update_attributes(params[:battle])
      redirect_to @battle, :flash => {:success => "You joined the battle."}
    else
      redirect_to root_path, :flash => {:error => "Failed to join battle"}
    end
  end
  
  def attack
    @battle = Battle.find(params[:id])
    
    if(current_user == @battle.starter) #only choose skill for the owner of the monster
      if(@battle.battle_turns.last.starter_skill_id == nil) #only choose the skill once
        if(@battle.starter_team_member.knows_skill(params[:skill])) #only use skills the monster knows
          @battle.battle_turns.last.update_attribute(:starter_skill_id, params[:skill].to_i)
          pusher_data = {:skill => Skill.find(params[:skill].to_i).name, :attacker => @battle.starter.id, :defender => @battle.opponent.id}
        end
      else
        multiple_skill_pick = true
      end
    elsif(current_user == @battle.opponent)
      if(@battle.battle_turns.last.opponent_skill_id == nil)
        if(@battle.opponent_team_member.knows_skill(params[:skill]))
          @battle.battle_turns.last.update_attribute(:opponent_skill_id, params[:skill].to_i)
          pusher_data = {:skill => Skill.find(params[:skill].to_i).name, :attacker => @battle.opponent.id, :defender => @battle.starter.id}
        end
      else
        multiple_skill_pick = true
      end
    end
    
    if(!@battle.winner)
      if(@battle.battle_turns.last.complete)
        Pusher["presence-battle-#{@battle.id}"].trigger('turn-attack', pusher_data)
        @battle.battle_turns.last.turn_over
      else
        if(multiple_skill_pick)
          # push alert to user saying to calm down
        else
          Pusher["presence-battle-#{@battle.id}"].trigger('turn-attack', pusher_data)
        end
      end
    else
      # there is a winner, notify the user...
    end
    
    respond_to do |format|  
      format.html { redirect_to @battle }
      format.js 
    end
  end
  
  def current_turn
    @battle = Battle.find(params[:id])
    (current_user == @battle.opponent) ? role = "opponent" : role = "starter"
    render :json => {:battle => @battle.battle_turns.last, :role => role}
  end
  
end
