class TamedMonstersController < ApplicationController
  before_filter :authenticate_user!, :only => [:new, :create]
  
  def index
    @tamed_monsters = TamedMonster.find(:all, :conditions => ["user_id == ?", current_user])
    if @tamed_monsters.empty?
      redirect_to welcome_path
      return
    end
    
    respond_to do |format|
      format.html
      format.json {render :json => @tamed_monsters}
    end
  end
  
  def show
    @tamed_monster = TamedMonster.find(params[:id])
    
    respond_to do |format|
      format.html
      format.json {render :json => @tamed_monster.as_json(:only => [:id, :name, :current_hp, :experience],
                                                          :include => {:user => {:only => [:id, :email, :nickname]},
                                                                       :monster => {:only => [:id, :name, :type_id]},
                                                                       :skill_1 => {:only => [:id, :name]},
                                                                       :skill_2 => {:only => [:id, :name]},
                                                                       :skill_3 => {:only => [:id, :name]},
                                                                       :skill_4 => {:only => [:id, :name]}
                                                                      },
                                                          :methods => :hp
                                                         )
                  }
      #wtf? @tamed_monster.as_json(:only => [:name], :include => {:user => {:only => [:id, :email, :nickname]}})
    end
  end
  
  def new
    if params[:monster]
      @monster = Monster.find(params[:monster])
    end
    
    if @monster.can_afford(current_user) == false
      if(@monster.starter == false && current_user.monsters.count == 0)
        redirect_to root_path, :flash => {:error => "Must be a starter"}
      elsif(current_user.monsters.count > 0)
        redirect_to root_path, :flash => {:error => "Not enough credits."}
      end
    end
    
    if @monster
      @tamed_monster = TamedMonster.new(:name => @monster.name, :monster => @monster)
    else
      redirect_to root_path
    end
  end

  def edit
    @tamed_monster = TamedMonster.find(params[:id])
  end
  
  def update
    @tamed_monster = TamedMonster.find(params[:id])

    if @tamed_monster.update_attributes(params[:tamed_monster])
      redirect_to @tamed_monster, :flash => {:success => "You updated your monster."}
    else
      render action: "edit"
    end
  end

  def create
    @tamed_monster = TamedMonster.new(params[:tamed_monster])
    @tamed_monster.user = current_user
    @tamed_monster.monster_id = params[:tamed_monster][:monster_id]

    if @tamed_monster.save
      redirect_to root_path, :flash => {:success => "Monster '#{@tamed_monster.name}' was successfully obtained."}
    else
      render action: "new"
    end
  end
  
end
