class BattleTurn < ActiveRecord::Base
  belongs_to :battle
  belongs_to :starter_skill, :class_name => 'Skill'
  belongs_to :opponent_skill, :class_name => 'Skill'
  
  def self.skill_history(battle_id, limit = 0)
    limit -= 1
    battle = Battle.find(battle_id)
    history = []
    skill_cache = {} #so we only hit the database when we need to
    ret = []
    
    battle.battle_turns.each do |turn|
      if(turn.starter_skill_id != nil && turn.opponent_skill_id != nil)
        if(!skill_cache[turn.starter_skill_id])
          skill_cache[turn.starter_skill_id] = {:id => turn.starter_skill_id, :name => turn.starter_skill.name}
        elsif(!skill_cache[turn.opponent_skill_id])
          skill_cache[turn.opponent_skill_id] = {:id => turn.opponent_skill_id, :name => turn.opponent_skill.name}
        end
      end
      history.push({:time_stamp => turn.created_at,
                    :starter_skill => skill_cache[turn.starter_skill_id],
                    :opponent_skill => skill_cache[turn.opponent_skill_id],
                    :starter_damage => turn.starter_damage,
                    :opponent_damage => turn.opponent_damage,
                    :outcome => turn.outcome})
    end
    history.sort! { |a, b| a[:time_stamp] <=> b[:time_stamp] }

    history.each_with_index do |history_item, index|
      ret.push({:turn_number => index + 1,
                :time_stamp => history_item[:time_stamp],
                :starter_skill => history_item[:starter_skill],
                :opponent_skill => history_item[:opponent_skill],
                :starter_damage => history_item[:starter_damage],
                :opponent_damage => history_item[:opponent_damage],
                :outcome => history_item[:outcome]})
    end
    
    ret.sort! { |a, b| b[:turn_number] <=> a[:turn_number] }
    ret.slice(0..limit)
  end
  
=begin  
  def self.formatted_skill_history(battle_id, limit = 0)
    battle = Battle.find(battle_id)
    history = self.skill_history(battle_id, limit)
    ret = []
    
    history.each do |history_item|
      ret.push("#{history_item[:time_stamp].strftime("%c")} - Turn: #{history_item[:turn_number]} - #{battle.starter.name} used #{history_item[:starter_skill][:name]} for")
    end
    
    ret
  end
=end
  
  def complete
    if(self.starter_skill && self.opponent_skill)
      true
    else
      false
    end
  end
  
  def turn_over
    if(self.complete)
      #do damage, send notifications...
      if self.battle.starter_team_member.speed > self.battle.opponent_team_member.speed
        result_1 = process_turn(self.battle.starter_team_member, self.battle.opponent_team_member, self.starter_skill)
        result_2 = process_turn(self.battle.opponent_team_member, self.battle.starter_team_member, self.opponent_skill)
      else
        result_1 = process_turn(self.battle.opponent_team_member, self.battle.starter_team_member, self.opponent_skill)
        result_2 = process_turn(self.battle.starter_team_member, self.battle.opponent_team_member, self.starter_skill)
      end

      if(result_1[:lost])
        #set status, award xp / credits... if lost is true then defender lost
        self.battle.update_attribute(:winner_id, result_1[:attacker].user.id)
        self.battle.update_attribute(:status, "#{result_1[:attacker].user.nickname} won!")
      elsif(result_2[:lost])
        #again, but for other player
        self.battle.update_attribute(:winner_id, result_2[:attacker].user.id)
        self.battle.update_attribute(:status, "#{result_2[:attacker].user.nickname} won!")
      else
        #self.battle.update_attribute(:status, "#{self.id}: #{result_1[:attacker].name} did #{result_1[:damage]} damage!  #{result_2[:attacker].name} did #{result_2[:damage]} damage!")
        self.battle.update_attribute(:status, self.outcome)
        BattleTurn.create(:battle_id => self.battle_id)
      end

      Pusher["presence-battle-#{self.battle.id}"].trigger('turn-over', {:last_turn => self.attributes, :battle => self.battle.attributes})
    end
  end
  
  private
  
    def process_turn(attacker, defender, ability)
      #damage = (( (2 * level + 10) / 250.0) * (attack / defense) * base + 2) * modifier
      if(Skill.switch == ability)
        #switch the monster
      else
        #what about skills that affect speed?
        #if(ability.effect == 'speed')
          #edit speed stat..speed stat on battle_turn?
        #elsif(ability.effect == 'damage')
        damage = ( (((2 * attacker.level + 10) / 250.0) * (attacker.attack / defender.defense) * ability.base_effect.abs + 2).round ) * modifier
      
        new_hp = defender.current_hp - damage
        defender.update_attribute(:current_hp, new_hp)
      end
      
      if(!self.outcome)
        last_turn = BattleTurn.skill_history(battle_id, limit = 1).first
        self.outcome = "Turn #{last_turn[:turn_number]}\n#{attacker.name} used #{ability.name} for #{damage} damage!\n"
      else
        self.outcome += "#{attacker.name} used #{ability.name} for #{damage} damage!\n"
      end
      self.update_attribute(:outcome, self.outcome)
      
      if(defender.current_hp <= 0) # not correct, need to check if all monsters on team are dead, not just the current...
        {:lost => true, :defender => defender, :attacker => attacker, :damage => damage}
      else
        {:lost => false, :defender => defender, :attacker => attacker, :damage => damage}
      end
    end
    
    def critical(odds = 5)
      if (rand(odds) < 1)
        2
      else
        1
      end
    end
    
    def modifier
      mod = critical * ((rand(15) + 86) / 100.0)
      
      if(mod <= 1)
        mod.ceil
      else
        mod.floor
      end
    end
    
end
