class TamedMonster < ActiveRecord::Base
  belongs_to :user
  belongs_to :monster
  belongs_to :skill_1, :class_name => "Skill"
  belongs_to :skill_2, :class_name => "Skill"
  belongs_to :skill_3, :class_name => "Skill"
  belongs_to :skill_4, :class_name => "Skill"
  
  has_many :battles
  
  validates :name, :presence => true
  validates :team_member,
            :numericality => { :greater_than_or_equal_to => 1,
                               :less_than_or_equal_to => 6 },
            :allow_blank => true,
            :uniqueness => { :scope => :user_id }
  validate :user_can_afford, :on => :create
  
  before_create :init
  
  searchable do
    text :name, :boost => 10
  end
  
  def to_param
    "#{id}-#{name.parameterize}"
  end
  
  def level
    (self.experience ** (1.0/3)).ceil
  end
  
  def hp
    ((((2 * self.monster.hp) + (self.ev_hp/4) + 100) * self.level) / 100) + 10
  end
  
  def attack
    ((((2 * self.monster.attack) + (self.ev_attack/4)) * self.level) / 100) + 5
  end
  
  def defense
    ((((2 * self.monster.defense) + (self.ev_defense/4)) * self.level) / 100) + 5
  end
  
  def speed
    ((((2 * self.monster.speed) + (self.ev_speed/4)) * self.level) / 100) + 5
  end
  
  def knows_skill(skill_id)
    skill_id = skill_id.to_i
    if(skill_id == skill_1_id || skill_id == skill_2_id || skill_id == skill_3_id || skill_id == skill_4_id)
      true
    else
      false
    end
  end
  
  def all_skills
    skills = []
    if(skill_1)
      skills.push(skill_1)
    end
    if(skill_2)
      skills.push(skill_2)
    end
    if(skill_3)
      skills.push(skill_3)
    end
    if(skill_4)
      skills.push(skill_4)
    end
    skills
  end
  
  private
  
    def init
      self.experience = 65
      self.ev_hp = 0
      self.ev_attack = 0
      self.ev_defense = 0
      self.ev_special_attack = 0
      self.ev_special_defense = 0
      self.ev_speed = 0
      self.current_hp = self.hp
      
      if self.user.tamed_monsters.count > 0
        credits = self.user.credits - self.monster.cost
        self.user.update_attribute(:credits, credits)
      else
        self.team_member = 1
      end
    end
    
    def user_can_afford
      if(self.monster.can_afford(self.user) == false && self.user.tamed_monsters.count > 0)
        errors.add('Cost:', "you can't afford #{self.monster.name}")
      end
    end
end
