class User < ActiveRecord::Base
  has_many :tamed_monsters
  has_many :monsters, :through => :tamed_monsters
  # Multiple foreign keys...
  has_many :starter_battles, :class_name => 'Battle', :foreign_key => 'starter_id'
  has_many :opponent_battles, :class_name => 'Battle', :foreign_key => 'opponent_id'
  has_many :won_battles, :class_name => 'Battle', :foreign_key => 'winner_id'
  def battles
    starter_battles + opponent_battles
  end
  
  
  # Include default devise modules. Others available are:
  # :token_authenticatable, :encryptable, :confirmable, :lockable, :timeoutable and :omniauthable
  devise :database_authenticatable, :registerable,
         :recoverable, :rememberable, :trackable, :validatable

  # Setup accessible (or protected) attributes for your model
  attr_accessible :email, :password, :password_confirmation, :remember_me, :first_name, :last_name, :experience, :credits, :nickname
  
  before_create :init
  
  validates :nickname, :presence => true,
                       :uniqueness => true
  
  searchable do
   text :nickname, :boost => 100
  end
  
  def to_param
    "#{id}-#{nickname.parameterize}"
  end
  
  def level
    (self.experience ** (1.0/3)).ceil
  end
  
  def exp_to_next_level
    self.level ** 3
  end
  
  def current_team
    team = []
    self.tamed_monsters.each do |tamed_monster|
      if(!tamed_monster.team_member.blank?)
        team.push(tamed_monster)
      end
    end
    
    team.sort!{|a, b| a.team_member <=> b.team_member }
  end
  
  private
  
    def init
      self.experience = 1
      self.credits = 0
    end
end
