//
//  BattleTableViewController.m
//  MonsterBattle
//
//  Created by Joshua Button on 11/25/11.
//  Copyright (c) 2011 CSU Chico. All rights reserved.
//

#import "BattleTableViewController.h"
#import "MonsterBattle.h"
#import "ASIHTTPRequest.h"
#import "BattleViewController.h"
#import "Battle.h"
#import "User.h"

@implementation BattleTableViewController
@synthesize model, battles;

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(battleDownloaded:) name:@"BattleDownloaded" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userDownloaded:) name:@"UserDownloaded" object:nil];
    
    self.model = [MonsterBattle sharedInstance];
    
    if(!self.battles) {
        NSString *stringURL = [NSString stringWithFormat:@"%@%@", kUrlHost, kUrlPathCurrentUserBattles];
        ASIHTTPRequest *request = [self.model getDataSynchronouslyFromURL:stringURL];
        NSData *responseData = [request responseData];
        self.battles = [NSJSONSerialization 
                        JSONObjectWithData:responseData
                        options:kNilOptions 
                        error:nil];
    }
    //NSLog(@"self.battles: %@", self.battles);

    for(NSDictionary *battle in self.battles) {
        NSNumber *identifier = [battle objectForKey:@"id"];
        [self.model battleFromCache:identifier];
    }
    
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"BattleDownloaded" object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"UserDownloaded" object:nil];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [self.battles count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"BattleTableViewCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
    // Configure the cell...
    
    NSNumber *identifier = [[self.battles objectAtIndex:indexPath.row] objectForKey:@"id"];
    Battle *battle = [self.model battleFromCache:identifier];
    if(battle.starterId != nil) {
        User *starter = [self.model userFromCache:battle.starterId];
        User *opponent = [self.model userFromCache:battle.opponentId];
        cell.textLabel.text = [NSString stringWithFormat:@"ID: %@ - %@ vs %@", battle.identifier, starter.nickname, opponent.nickname];
    }
    
    
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller.
    /*
     <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
     [self.navigationController pushViewController:detailViewController animated:YES];
     [detailViewController release];
     */
}

- (void)battleDownloaded:(NSNotification *)notification
{
    //NSLog(@"Downloaded object: %@", [notification.object objectForKey:@"battle"]);
    [self.tableView reloadData];
}

- (void)userDownloaded:(NSNotification *)notification
{
    [self.tableView reloadData];
    //[self.model speciesFromCache:[[notification.object objectForKey:@"id"]
}


#pragma mark - Segue

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if([[segue identifier] isEqualToString:@"SelectedBattle"]) {
        BattleViewController *controller = [segue destinationViewController];
        NSUInteger selected = [[self.tableView indexPathForSelectedRow] row];
        //[controller setBattle:[self.battles objectAtIndex:selected]];
        
        NSNumber *battleId = [[self.battles objectAtIndex:selected] objectForKey:@"id"];
        [controller setBattle:[self.model battleFromCache:battleId]];
        
        //NSNumber *battleId = [[self.battles objectAtIndex:selected] objectForKey:@"id"];
        //NSArray *battle = [model.battleCache objectForKey:@"current_team"];
        //[controller setCreatures:creatures];
    }
}

@end
