//
//  BattleViewController.m
//  MonsterBattle
//
//  Created by Joshua Button on 11/25/11.
//  Copyright (c) 2011 CSU Chico. All rights reserved.
//

#import "BattleViewController.h"
#import "MonsterBattle.h"
#import "ASIHTTPRequest.h"
#import "AsyncImageView.h"
#import "Monster.h"
#import "Species.h"
#import "User.h"
#import "Battle.h"
#import "BattleSkillViewController.h"
#import "ASIDownloadCache.h"

@implementation BattleViewController
@synthesize model, starter, opponent, starterTeamMember, opponentTeamMember, battle, reloadTimer, battlefieldImageView, starterGravatarImageView, starterMonsterImageView, starterMonsterHealth, starterName, starterTeamMemberName, opponentGravatarImageView, opponentMonsterImageView, opponentMonsterHealth, opponentName, opponentTeamMemberName;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView
{
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userDownloaded:) name:@"UserDownloaded" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(monsterDownloaded:) name:@"MonsterDownloaded" object:nil];
    
    self.model = [MonsterBattle sharedInstance];
    
    //self.starter = [[User alloc] initWithId:self.battle.starterId];
    //self.opponent = [[User alloc] initWithId:self.battle.opponentId];
    self.starter = [model userFromCache:self.battle.starterId];
    self.opponent = [model userFromCache:self.battle.opponentId];
    self.starterTeamMember = [model monsterFromCache:self.battle.starterTeamMemberId];
    self.opponentTeamMember = [model monsterFromCache:self.battle.opponentTeamMemberId];
    
    [self reloadData];
    /* get rid of notification post on cache and put and update function call here...need a way to refresh the data... */
    
    //NSLog(@"Battle: %@, Starter: %@, Cache: %@", self.battle, starter, model.userCache);
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"MonsterDownloaded" object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"UserDownloaded" object:nil];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    self.reloadTimer = [NSTimer scheduledTimerWithTimeInterval:10.0f 
                                                        target:self
                                                      selector:@selector(fireTimer:) 
                                                      userInfo:nil
                                                       repeats:YES];
    [self.reloadTimer fire];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [self.reloadTimer invalidate];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
/* get rid of these? */
- (void)userDownloaded:(NSNotification *)notification
{
    if([(User *)notification.object identifier] == battle.starterId) {
        self.starterName.text = [(User *)notification.object nickname];
        [self.starterGravatarImageView downloadGravatar:[(User *)notification.object gravatarHash]];
    } else {
        self.opponentName.text = [(User *)notification.object nickname];
        [self.opponentGravatarImageView downloadGravatar:[(User *)notification.object gravatarHash]];
    }
}

- (void)monsterDownloaded:(NSNotification *)notification
{
    if([(Monster *)notification.object identifier] == battle.starterTeamMemberId) {
        [self.starterMonsterImageView downloadFromPath:[[(Monster *)notification.object species] backImageViewPath]];
        self.starterTeamMemberName.text = [(Monster *)notification.object name];
        [self.starterMonsterHealth setProgress:([[(Monster *)notification.object currentHp] floatValue] / [[(Monster *)notification.object hp] floatValue])];
    } else {
        [self.opponentMonsterImageView downloadFromPath:[[(Monster *)notification.object species] frontImageViewPath]];
        self.opponentTeamMemberName.text = [(Monster *)notification.object name];
        [self.opponentMonsterHealth setProgress:([[(Monster *)notification.object currentHp] floatValue] / [[(Monster *)notification.object hp] floatValue])];
    }
}

- (void)reloadData
{
    self.title = [NSString stringWithFormat:@"%@ vs %@", self.starter.nickname, self.opponent.nickname];
    
    [self.battlefieldImageView downloadFromPath:@"/battle_background_grass.png"];
    
    self.starterName.text = [self.starter nickname];
    self.opponentName.text = [self.opponent nickname];
    [self.starterGravatarImageView downloadGravatar:[self.starter gravatarHash]];
    [self.opponentGravatarImageView downloadGravatar:[self.opponent gravatarHash]];
    
    [self.starterMonsterImageView downloadFromPath:[[self.starterTeamMember species] backImageViewPath]];
    [self.opponentMonsterImageView downloadFromPath:[[self.opponentTeamMember species] frontImageViewPath]];
    self.starterTeamMemberName.text = [self.starterTeamMember name];
    self.opponentTeamMemberName.text = [self.opponentTeamMember name];
    
    [self.starterMonsterHealth setProgress:([[self.starterTeamMember currentHp] floatValue] / [[self.starterTeamMember hp] floatValue])];
    [self.opponentMonsterHealth setProgress:([[self.opponentTeamMember currentHp] floatValue] / [[self.opponentTeamMember hp] floatValue])];
}

- (void)fireTimer:(NSTimer *)theTimer
{
    NSLog(@"Timer fired!");
    
    self.starter = [model userFromCache:self.battle.starterId];
    self.opponent = [model userFromCache:self.battle.opponentId];
    self.starterTeamMember = [model monsterFromCache:self.battle.starterTeamMemberId];
    self.opponentTeamMember = [model monsterFromCache:self.battle.opponentTeamMemberId];
    
    [self.starterTeamMember refresh];
    [self.opponentTeamMember refresh];
    
    [self reloadData];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if([[segue identifier] isEqualToString:@"SkillsPressed"]) {
        BattleSkillViewController *controller = [segue destinationViewController];
        [controller setBattle:self.battle];
        /* send the right monster so we cna list the right skills */

        if([self.battle.role isEqualToString:@"starter"])
            [controller setMonster:self.starterTeamMember];
        else
            [controller setMonster:self.opponentTeamMember];
    }
}

@end
