//
//  CreatureTableViewController.m
//  MonsterBattle
//
//  Created by Joshua Button on 11/20/11.
//  Copyright (c) 2011 CSU Chico. All rights reserved.
//

#import "CreatureTableViewController.h"
#import "MonsterBattle.h"
#import "ASIHTTPRequest.h"
#import "CreatureTableViewCell.h"
#import "AsyncImageView.h"
#import "MonsterViewController.h"
#import "Species.h"
#import "Monster.h"


@implementation CreatureTableViewController
@synthesize model, creatures;

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.model = [MonsterBattle sharedInstance];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reloadTable) name:@"SpeciesDownloaded" object:nil];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"SpeciesDownloaded" object:nil];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [creatures count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
#warning this needs to be redone
    /*
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }*/
    
    // Configure the cell...
    //cell.textLabel.text = [[creatures objectAtIndex:indexPath.row] objectForKey:@"name"];
    
    CreatureTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"CreatureTableViewCell"];
    
    //NSMutableDictionary *data = [self.model speciesCache];
    //NSNumber *identifier = [[creatures objectAtIndex:indexPath.row] objectForKey:@"monster_id"];
    /*NSMutableDictionary *species = [[self.model speciesCache] objectForKey:identifier];
    cell.titleLabel.text = [[creatures objectAtIndex:indexPath.row] objectForKey:@"name"];
    cell.subtitleLabel.text = [species objectForKey:@"name"];
    [cell.creatureImageView downloadCreature:identifier];*/
    
    NSNumber *identifier = [[creatures objectAtIndex:indexPath.row] objectForKey:@"id"];
    Monster *monster = [self.model monsterFromCache:identifier];
    cell.titleLabel.text = monster.name;
    cell.subtitleLabel.text = monster.species.name;
    [cell.creatureImageView downloadFromPath:monster.species.frontImageViewPath];
    
    return cell;
}

/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller.
    /*
     <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
     [self.navigationController pushViewController:detailViewController animated:YES];
     [detailViewController release];
     */
    
    NSNumber *identifier = [[self.creatures objectAtIndex:indexPath.row]  objectForKey:@"id"];
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"MainStoryboard" bundle:nil];
    MonsterViewController *monsterController = [storyboard instantiateViewControllerWithIdentifier:@"MonsterView"];
    monsterController.monster = [self.model monsterFromCache:identifier];
    [self.navigationController pushViewController:monsterController animated:YES];
}

- (void)reloadTable
{
    [self.tableView reloadData];
}

#pragma mark - ASIHTTPRequest Delegates

- (void)requestFinished:(ASIHTTPRequest *)request
{
    NSData *responseData = [request responseData];
    NSDictionary *species = [NSJSONSerialization 
                             JSONObjectWithData:responseData
                             options:kNilOptions 
                             error:nil];
    [model.speciesCache setObject:species forKey:[species objectForKey:@"id"]];
    [self.tableView reloadData];
}

- (void)requestFailed:(ASIHTTPRequest *)request
{
    NSError *error = [request error];
    NSLog(@"Creature Table View Download Error: %@", error);
}

@end
