//
//  Monster.m
//  MonsterBattle
//
//  Created by Joshua Button on 11/26/11.
//  Copyright (c) 2011 CSU Chico. All rights reserved.
//

#import "Monster.h"
#import "MonsterBattle.h"
#import "ASIHTTPRequest.h"
#import "Species.h"

@implementation Monster
@synthesize identifier, name, currentHp, hp, experience, species, userId, skillOne, skillTwo, skillThree, skillFour;

- (id)initWithId:(NSNumber *)ident
{
    NSLog(@"Monster init with identifer: %@", ident);
    self = [super init];
    if(self) {
        self.identifier = ident;
        [self refresh];
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"SpeciesDownloaded" object:self.species];
}

- (void)speciesDownloaded:(NSNotification *)notification
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"MonsterDownloaded" object:self];
}

- (void)refresh
{
    NSString *hostPath = [NSString stringWithFormat:@"%@%@", kUrlHost, kUrlPathMonsters];
    NSString *stringURL = [NSString stringWithFormat:hostPath, self.identifier];
    NSURL *url = [NSURL URLWithString:stringURL];
    
    __block ASIHTTPRequest *request = [ASIHTTPRequest requestWithURL:url];
    [request setCompletionBlock:^{
        NSData *responseData = [request responseData];
        NSDictionary *monster = [NSJSONSerialization 
                                 JSONObjectWithData:responseData
                                 options:kNilOptions 
                                 error:nil];
        
        self.name = [monster objectForKey:@"name"];
        self.currentHp = [monster objectForKey:@"current_hp"];
        self.hp = [monster objectForKey:@"hp"];
        self.experience = [monster objectForKey:@"experience"];
        self.species = [[Species alloc] initWithId:[[monster objectForKey:@"monster"] objectForKey:@"id"]];
        self.userId = [[monster objectForKey:@"user"] objectForKey:@"id"];
        self.skillOne = [monster objectForKey:@"skill_1"];
        self.skillTwo = [monster objectForKey:@"skill_2"];
        self.skillThree = [monster objectForKey:@"skill_3"];
        self.skillFour = [monster objectForKey:@"skill_4"];
        
        NSLog(@"Downloaded monster %@", self.name);
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(speciesDownloaded:) name:@"SpeciesDownloaded" object:self.species];
    }];
    [request setFailedBlock:^{
        NSError *error = [request error];
        NSLog(@"Monster Download Error: %@", error);
    }];
    [request startAsynchronous];
}

@end