//
//  MonsterBattle.m
//  MonsterBattle
//
//  Created by Joshua Button on 11/20/11.
//  Copyright (c) 2011 CSU Chico. All rights reserved.
//

#import "MonsterBattle.h"
#import "ASIHTTPRequest.h"
#import "ASIDownloadCache.h"
#import "Battle.h"
#import "User.h"
#import "Monster.h"
#import "Species.h"

@implementation MonsterBattle
@synthesize currentUser, speciesCache, monsterCache, battleCache, userCache;

+ (MonsterBattle *)sharedInstance
{
    static MonsterBattle *sharedInstance = nil;
    static dispatch_once_t once = 0;
    dispatch_once(&once, ^{
        sharedInstance = [[MonsterBattle alloc] init];
        sharedInstance.speciesCache = [[NSMutableDictionary alloc] init];
        sharedInstance.monsterCache = [[NSMutableDictionary alloc] init];
        sharedInstance.battleCache = [[NSMutableDictionary alloc] init];
        sharedInstance.userCache = [[NSMutableDictionary alloc] init];
        [ASIHTTPRequest setDefaultCache:[ASIDownloadCache sharedCache]];
        //[[ASIDownloadCache sharedCache] setShouldRespectCacheControlHeaders:NO];
        //[[ASIDownloadCache sharedCache] setDefaultCachePolicy:ASIAskServerIfModifiedCachePolicy];
    });
    return sharedInstance;
}

- (ASIHTTPRequest *)getDataSynchronouslyFromURL:(NSString *)stringURL
{
    NSURL *url = [NSURL URLWithString:stringURL];
    ASIHTTPRequest *request = [ASIHTTPRequest requestWithURL:url];
    [request setAuthenticationScheme:(NSString *)kCFHTTPAuthenticationSchemeBasic];
    //request.shouldPresentAuthenticationDialog = true;
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSString *username = [defaults stringForKey:@"email"];
    NSString *password = [defaults stringForKey:@"password"]; // not very secure...
    NSLog(@"Attempting to log in - Username: %@", username);
    
    [request setUsername:username];
    [request setPassword:password];
    [request startSynchronous];
    
    return request;
}

- (NSError *)logInAndGetCurrentUser
{
    ASIHTTPRequest *request = [self getDataSynchronouslyFromURL:[NSString stringWithFormat:@"%@%@", kUrlHost, kUrlPathCurrentUser]];
    
    NSError *error = [request error];
    if (error) {
        NSLog(@"logInAndGetCurrentUser - error: %@", error);
    } else {
        NSData *responseData = [request responseData];
        [self setCurrentUser:[NSJSONSerialization 
                              JSONObjectWithData:responseData
                              options:kNilOptions 
                              error:nil]];
        NSLog(@"logInAndGetCurrentUser - currentUser: %@", self.currentUser);
    }
    
    return error;
}

- (Species *)speciesFromCache:(NSNumber *)identifier
{
    if(![self.speciesCache objectForKey:identifier]) {
        Species *species = [[[Species alloc] initWithId:identifier] autorelease];
        [self.speciesCache setObject:species forKey:identifier];
    } else {
        //[[NSNotificationCenter defaultCenter] postNotificationName:@"SpeciesDownloaded" object:[self.speciesCache objectForKey:identifier]];
    }
    return [self.speciesCache objectForKey:identifier];
}

- (Monster *)monsterFromCache:(NSNumber *)identifier
{
    if(![self.monsterCache objectForKey:identifier]) {
        Monster *monster = [[[Monster alloc] initWithId:identifier] autorelease];
        [self.monsterCache setObject:monster forKey:identifier];
    } else {
        //[[NSNotificationCenter defaultCenter] postNotificationName:@"MonsterDownloaded" object:[self.monsterCache objectForKey:identifier]];
    }
    return [self.monsterCache objectForKey:identifier];
}

- (Battle *)battleFromCache:(NSNumber *)identifier
{
    if(![self.battleCache objectForKey:identifier]) {
        Battle *battle = [[[Battle alloc] initWithId:identifier] autorelease];
        [self.battleCache setObject:battle forKey:identifier];
    }
    return [self.battleCache objectForKey:identifier];
}

- (User *)userFromCache:(NSNumber *)identifier
{
    if(![self.userCache objectForKey:identifier]) {
        User *user = [[[User alloc] initWithId:identifier] autorelease];
        [self.userCache setObject:user forKey:identifier];
    } else {
        //[[NSNotificationCenter defaultCenter] postNotificationName:@"UserDownloaded" object:[self.userCache objectForKey:identifier]];
    }
    return [self.userCache objectForKey:identifier];
}

- (float)calculateLevelCap:(float)level
{
    float levelCap = powf(level, 3);
    return levelCap;
}

- (float)calculateExperienceBarWithExp:(float)experience Level:(float)level
{
    float currentProgress = experience / [self calculateLevelCap:level];
    return currentProgress;
}
@end
