//
//  SearchViewController.m
//  MonsterBattle
//
//  Created by Joshua Button on 11/29/11.
//  Copyright (c) 2011 CSU Chico. All rights reserved.
//

#import "SearchViewController.h"
#import "MonsterBattle.h"
#import "ASIHTTPRequest.h"
#import "SpeciesViewController.h"
#import "MonsterViewController.h"
#import "User.h"


@implementation SearchViewController
@synthesize model, searchBar, searchResults, pageNumber, numberOfResults;

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reloadTable:) name:@"SearchDownloaded" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reloadTable:) name:@"UserDownloaded" object:nil];
    
    self.model = [MonsterBattle sharedInstance];
    self.searchBar.text = @"";
    self.pageNumber = 1;
    self.searchResults = nil;
    [self search];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"SearchDownloaded" object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"UserDownloaded" object:nil];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    /* Is self.numberOfResults == 0 then we are at the end of the search results,
     stop trying to load more */
    return self.searchResults && self.numberOfResults != 0 ? self.searchResults.count + 1 : self.searchResults.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"SearchResultCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
    }
    
    if(indexPath.row == self.searchResults.count) {
        cell.accessoryType = UITableViewCellAccessoryNone;
        cell.detailTextLabel.text = @"";
        cell.textLabel.text = @"Loading...";
    } else {
        cell.accessoryType = UITableViewCellAccessoryDetailDisclosureButton;
        NSString *class = [[self.searchResults objectAtIndex:indexPath.row] objectForKey:@"class"];
        NSDictionary *result = [[self.searchResults objectAtIndex:indexPath.row] objectForKey:@"result"];
        if([class isEqualToString:@"Monster"]) {
            User *user = [self.model userFromCache:[result objectForKey:@"user_id"]];
            cell.textLabel.text = [result objectForKey:@"name"];
            cell.detailTextLabel.text = [NSString stringWithFormat:@"Monster - Owner: %@", user.nickname];
        } else if([class isEqualToString:@"Species"]) {
            cell.textLabel.text = [result objectForKey:@"name"];
            cell.detailTextLabel.text = class; //try to get type here
        } else if([class isEqualToString:@"User"]) {
            cell.textLabel.text = [result objectForKey:@"nickname"];
            cell.detailTextLabel.text = class;
        }
    }
    
    return cell;
}

- (void)reloadTable:(NSNotification *)notification
{
    [self.tableView reloadData];
}

/* this will just reuse the text currently in the search bar, may be a bug if text is changed without searching and load more is used */
- (void)search
{
    NSString *hostPath = [NSString stringWithFormat:@"%@%@", kUrlHost, kUrlPathSearch];
    NSString *searchURL = [NSString stringWithFormat:hostPath, self.searchBar.text];
    NSString *stringURL = [NSString stringWithFormat:@"%@&page=%d", searchURL, self.pageNumber];
    NSURL *url = [NSURL URLWithString:stringURL];
    
    __block ASIHTTPRequest *request = [ASIHTTPRequest requestWithURL:url];
    [request setCompletionBlock:^{
        NSData *responseData = [request responseData];
        NSArray *results = [NSJSONSerialization 
                            JSONObjectWithData:responseData
                            options:kNilOptions 
                            error:nil];
        
        self.numberOfResults = [results count];
        if(!self.searchResults) {
            self.searchResults = results;
        } else {
            self.searchResults = [self.searchResults arrayByAddingObjectsFromArray:results];
        }
        
        [[NSNotificationCenter defaultCenter] postNotificationName:@"SearchDownloaded" object:self.searchResults];
    }];
    [request setFailedBlock:^{
        NSError *error = [request error];
        NSLog(@"Search Download Error: %@", error);
    }];
    [request startAsynchronous];
    
    [self.searchBar resignFirstResponder];
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"MainStoryboard" bundle:nil];
    NSString *class = [[self.searchResults objectAtIndex:indexPath.row] objectForKey:@"class"];
    NSNumber *identifier = [[[self.searchResults objectAtIndex:indexPath.row] objectForKey:@"result"] objectForKey:@"id"];
    if([class isEqualToString:@"Species"]) {
        SpeciesViewController *speciesController = [storyboard instantiateViewControllerWithIdentifier:@"SpeciesView"];
        speciesController.species = [self.model speciesFromCache:identifier];
        [self.navigationController pushViewController:speciesController animated:YES];
    } else if([class isEqualToString:@"Monster"]) {
        MonsterViewController *monsterController = [storyboard instantiateViewControllerWithIdentifier:@"MonsterView"];
        monsterController.monster = [self.model monsterFromCache:identifier];
        [self.navigationController pushViewController:monsterController animated:YES];
    }
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if(indexPath.row == self.searchResults.count) {
        NSLog(@"Load more");
        self.pageNumber += 1;
        [self search];
    }
}

#pragma mark - Search bar delegate

- (void)searchBarSearchButtonClicked:(UISearchBar *)sb
{
    self.pageNumber = 1;
    self.searchResults = nil;
    [self search];
}

- (void)searchBarCancelButtonClicked:(UISearchBar *)sb
{
    [self.searchBar resignFirstResponder];
}

@end
