//
//  Species.m
//  MonsterBattle
//
//  Created by Joshua Button on 11/26/11.
//  Copyright (c) 2011 CSU Chico. All rights reserved.
//

#import "Species.h"
#import "MonsterBattle.h"
#import "ASIHTTPRequest.h"
#import "AsyncImageView.h"

@implementation Species
@synthesize identifier, name, description, hp, attack, defense, specialAttack, specialDefense, speed, cost, frontImageViewPath, backImageViewPath;

- (id)initWithId:(NSNumber *)ident
{
    self = [super init];
    if(self) {
        NSString *hostPath = [NSString stringWithFormat:@"%@%@", kUrlHost, kUrlPathSpecies];
        NSString *stringURL = [NSString stringWithFormat:hostPath, ident];
        NSURL *url = [NSURL URLWithString:stringURL];
        
        __block ASIHTTPRequest *request = [ASIHTTPRequest requestWithURL:url];
        [request setCompletionBlock:^{
            NSData *responseData = [request responseData];
            NSDictionary *species = [NSJSONSerialization 
                                     JSONObjectWithData:responseData
                                     options:kNilOptions 
                                     error:nil];
            
            self.identifier = [species objectForKey:@"id"];
            self.name = [species objectForKey:@"name"];
            self.description = [species objectForKey:@"description"];
            self.hp = [species objectForKey:@"hp"];
            self.attack = [species objectForKey:@"attack"];
            self.defense = [species objectForKey:@"defense"];
            self.specialAttack = [species objectForKey:@"special_attack"];
            self.specialDefense = [species objectForKey:@"special_defense"];
            self.speed = [species objectForKey:@"speed"];
            self.cost = [species objectForKey:@"cost"];
            self.frontImageViewPath = [[species objectForKey:@"get_images"] objectForKey:@"front"];
            self.backImageViewPath = [[species objectForKey:@"get_images"] objectForKey:@"back"];
            
            [[NSNotificationCenter defaultCenter] postNotificationName:@"SpeciesDownloaded" object:self];
            
            NSLog(@"Downloaded species %@", self.name);
        }];
        [request setFailedBlock:^{
            NSError *error = [request error];
            NSLog(@"Species Download Error: %@", error);
        }];
        [request startAsynchronous];
    }
    return self;
}
@end
