//
//  User.m
//  MonsterBattle
//
//  Created by Joshua Button on 11/26/11.
//  Copyright (c) 2011 CSU Chico. All rights reserved.
//

#import "User.h"
#import "MonsterBattle.h"
#import "ASIHTTPRequest.h"

@implementation User
@synthesize identifier, email, nickname, firstName, lastName, level, expToNextLevel, gravatarHash;

- (id)initWithId:(NSNumber *)ident
{
    self = [super init];
    if(self) {
        [self downloadWithId:ident];
    }
    return self;
}

- (void)downloadWithId:(NSNumber *)ident
{
    NSString *hostPath = [NSString stringWithFormat:@"%@%@", kUrlHost, kUrlPathProfile];
    NSString *stringURL = [NSString stringWithFormat:hostPath, ident];
    NSURL *url = [NSURL URLWithString:stringURL];
    
    __block ASIHTTPRequest *request = [ASIHTTPRequest requestWithURL:url];
    [request setCompletionBlock:^{
        NSData *responseData = [request responseData];
        NSDictionary *profile = [NSJSONSerialization 
                                 JSONObjectWithData:responseData
                                 options:kNilOptions 
                                 error:nil];
        
        self.identifier = [profile objectForKey:@"id"];
        self.email = [profile objectForKey:@"email"];
        self.nickname = [profile objectForKey:@"nickname"];
        self.firstName = [profile objectForKey:@"first_name"];
        self.lastName = [profile objectForKey:@"last_name"];
        self.level = [profile objectForKey:@"level"];
        self.expToNextLevel = [profile objectForKey:@"exp_to_next_level"];
        self.gravatarHash = [profile objectForKey:@"gravatar"];
        
        [[NSNotificationCenter defaultCenter] postNotificationName:@"UserDownloaded" object:self];
        
        NSLog(@"Downloaded user %@", self.nickname);
    }];
    [request setFailedBlock:^{
        NSError *error = [request error];
        NSLog(@"User Download Error: %@", error);
    }];
    [request startAsynchronous];
}

@end
